/*!
 *
 * A base set of easing methods
 * Most of which were found here:
 * https://gist.github.com/gre/1650294
 *
 * @Easing
 * @author: kitajchuk
 *
 */
(function ( factory ) {
    
    if ( typeof exports === "object" && typeof module !== "undefined" ) {
        module.exports = factory();

    } else if ( typeof window !== "undefined" ) {
        window.Easing = factory();
    }
    
})(function () {

    /**
     *
     * Easing functions
     * @namespace Easing
     * @memberof! <global>
     *
     */
    var Easing = {
        /**
         *
         * Produce a linear ease
         * @method linear
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        linear: function ( t ) { return t; },
        
        /**
         *
         * Produce a swing ease like in jQuery
         * @method swing
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        swing: function ( t ) { return (1-Math.cos( t*Math.PI ))/2; },
        
        /**
         *
         * Accelerating from zero velocity
         * @method easeInQuad
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeInQuad: function ( t ) { return t*t; },
        
        /**
         *
         * Decelerating to zero velocity
         * @method easeOutQuad
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeOutQuad: function ( t ) { return t*(2-t); },
        
        /**
         *
         * Acceleration until halfway, then deceleration
         * @method easeInOutQuad
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeInOutQuad: function ( t ) { return t<0.5 ? 2*t*t : -1+(4-2*t)*t; },
        
        /**
         *
         * Accelerating from zero velocity
         * @method easeInCubic
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeInCubic: function ( t ) { return t*t*t; },
        
        /**
         *
         * Decelerating to zero velocity
         * @method easeOutCubic
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeOutCubic: function ( t ) { return (--t)*t*t+1; },
        
        /**
         *
         * Acceleration until halfway, then deceleration
         * @method easeInOutCubic
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeInOutCubic: function ( t ) { return t<0.5 ? 4*t*t*t : (t-1)*(2*t-2)*(2*t-2)+1; },
        
        /**
         *
         * Accelerating from zero velocity
         * @method easeInQuart
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeInQuart: function ( t ) { return t*t*t*t; },
        
        /**
         *
         * Decelerating to zero velocity
         * @method easeOutQuart
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeOutQuart: function ( t ) { return 1-(--t)*t*t*t; },
        
        /**
         *
         * Acceleration until halfway, then deceleration
         * @method easeInOutQuart
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeInOutQuart: function ( t ) { return t<0.5 ? 8*t*t*t*t : 1-8*(--t)*t*t*t; },
        
        /**
         *
         * Accelerating from zero velocity
         * @method easeInQuint
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeInQuint: function ( t ) { return t*t*t*t*t; },
        
        /**
         *
         * Decelerating to zero velocity
         * @method easeOutQuint
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeOutQuint: function ( t ) { return 1+(--t)*t*t*t*t; },
        
        /**
         *
         * Acceleration until halfway, then deceleration
         * @method easeInOutQuint
         * @param {number} t Difference in time
         * @memberof Easing
         * @returns a new t value
         *
         */
        easeInOutQuint: function ( t ) { return t<0.5 ? 16*t*t*t*t*t : 1+16*(--t)*t*t*t*t; }
    };
    
    
    return Easing;


});